/*
KURWICA v0.1 (C) 2019 IMI Kurwica
Enables installation of ESU on all editions of Windows 7

This is basically an "injector" utilizing a custom Application Verifier provider.
*/

#include <windows.h>
#include "internal.h"

static RTL_VERIFIER_THUNK_DESCRIPTOR avrfThunksSlc[2];
static RTL_VERIFIER_THUNK_DESCRIPTOR avrfThunksKrnl32[2];
static RTL_VERIFIER_DLL_DESCRIPTOR avrfDlls[3];
static RTL_VERIFIER_PROVIDER_DESCRIPTOR g_avrfProvider;

HRESULT WINAPI SLGetWindowsInformationDWORDHook(
	PCWSTR pwszValueName,
	PDWORD pdwValue
) {
	*pdwValue = 1;
	return S_OK;
}

BOOL WINAPI GetProductInfoHook(
	DWORD dwOSMajorVersion,
	DWORD dwOSMinorVersion,
	DWORD dwSpMajorVersion,
	DWORD dwSpMinorVersion,
	PDWORD pdwReturnedProductType
) {
	RTL_OSVERSIONINFOEXW VersionInfo;
	VersionInfo.dwOSVersionInfoSize = sizeof(RTL_OSVERSIONINFOEXW);
	RtlGetVersion(&VersionInfo);

	if(VersionInfo.wProductType == VER_NT_WORKSTATION) {
		// Windows Client - Professional
		*pdwReturnedProductType = 48;
	} else {
		// Assume Windows Server - ServerStandard
		*pdwReturnedProductType = 7;
	}

	return TRUE;
}

VOID RegisterProvider(VOID) {
	// slc.dll
	avrfThunksSlc[0].ThunkName = "SLGetWindowsInformationDWORD";
	avrfThunksSlc[0].ThunkOldAddress = NULL;
	avrfThunksSlc[0].ThunkNewAddress = &SLGetWindowsInformationDWORDHook;

	avrfDlls[0].DllName = L"slc.dll";
	avrfDlls[0].DllFlags = 0;
	avrfDlls[0].DllAddress = NULL;
	avrfDlls[0].DllThunks = avrfThunksSlc;

	// kernel32.dll
	avrfThunksKrnl32[0].ThunkName = "GetProductInfo";
	avrfThunksKrnl32[0].ThunkOldAddress = NULL;
	avrfThunksKrnl32[0].ThunkNewAddress = &GetProductInfoHook;

	avrfDlls[1].DllName = L"kernel32.dll";
	avrfDlls[1].DllFlags = 0;
	avrfDlls[1].DllAddress = NULL;
	avrfDlls[1].DllThunks = avrfThunksKrnl32;

	RtlSecureZeroMemory(&g_avrfProvider, sizeof(RTL_VERIFIER_PROVIDER_DESCRIPTOR));
	g_avrfProvider.Length = sizeof(RTL_VERIFIER_PROVIDER_DESCRIPTOR);
	g_avrfProvider.ProviderDlls = avrfDlls;
}

BOOL WINAPI InitRoutine(
	PVOID DllHandle,
	DWORD fdwReason,
	PRTL_VERIFIER_PROVIDER_DESCRIPTOR* pVPD
) {
	switch(fdwReason) {
		case DLL_PROCESS_ATTACH:
			LdrDisableThreadCalloutsForDll(DllHandle);
			break;

		case DLL_PROCESS_VERIFIER:
			RegisterProvider();
			*pVPD = &g_avrfProvider;
			break;

		default:
			break;
	}

	return TRUE;
}
